Class: CanvasSync::JobLog

Inherits:
ApplicationRecord
  • Object
show all
Defined in:
app/models/canvas_sync/job_log.rb

Overview

When you write a job that inherits from CanvasSync::Job it will log to the canvas_sync_job_logs table

Use this model to track failures, job run times, and metadata about a job.